const Path = require("path");
const fs = require("fs-extra");
const _ = require("lodash");

const log = require("@ombiel/aek-cli-support/logger");
const auth = require("@ombiel/aek-cli-support/auth");
const yarn = require("@ombiel/aek-cli-support/yarn");
const {FriendlyError} = require("@ombiel/aek-cli-support/nice-error");
const pullPackage = require("../pull-package");

const boilerplates = {
  "AEK Project": "aek-boilerplate",
  "TileSDK Project": "cm-tile-boilerplate",
};

module.exports = {
  name: "create",
  help: "create a new package",
  options: [
    {
      name: "username",
      message: "Please enter your App Manager username",
      noPrompt: true,
    },
    {
      name: "password",
      message: "Please enter your App Manager password",
      inputType: "password",
      noPrompt: true,
    },
    {
      name: "name",
      position: 1,
      help: "Name of package",
      message: "Name of Package",
      required: true,
      promptIfMissing: true,
    },
    {
      name: "boilerplate",
      abbr: "b",
      help: "Name of boilerplate",
      message: "Please select a boilerplate for your new project",
      choices: _.keys(boilerplates),
      required: true,
      promptIfMissing: true,
      choiceIsOpen: true,
    },
    {
      name: "devHost",
      help: "default hostname to run your dev server under (can be modified later in package.json)",
      message: "Please specify a hostname to run your dev server against",
      default: "bigskyu.ombiel.co.uk",
      beforeInquiry: auth.getOptionHostnames,
      promptIfMissing: true,
    },
  ],
  callback: function (options) {

    return auth.getUserDetails(options)
    .then(function (userDetails) {
      const packageName = options.name;
      // validate package name
      if (!(/^[a-z0-9-]{6,}$/).test(packageName)) {
        throw FriendlyError("Sorry, this is not a valid package name. Please use at least 6 lowercase characters, hyphens or numbers. Do not use uppercase characters these are not accepted.");
      }
      if (userDetails.apps !== "all") {
        const nspace = packageName.split("-")[0];
        if (_.includes(["aek","omb","ombiel","exlib","exlibris"],nspace)) {
          throw FriendlyError("Sorry, this is a reserved namespace, please try a different name");
        }
      }

      const boilerplateName = boilerplates[options.boilerplate] || options.boilerplate;

      const pullOptions = {package: boilerplateName};
      if (pullOptions.package.indexOf("@") !== 0) {
        pullOptions.package = `@ombiel/${pullOptions.package}`;
      }

      const destination = `./${packageName}`;
      pullOptions.destination = destination;

      return pullPackage(pullOptions).then(()=>{
        log.info(`Successfully extracted boilerplate (${pullOptions.package})`);
        return fs.readJSON(Path.resolve(destination,"package.json"));
      })
      .then((pkgjson)=>{

        const ombData = {
          boilerplate: `${pkgjson.name}@${pkgjson.version}`,
          access: {
            write: {
              users: [userDetails.username]
            }
          },
          settings: pkgjson.ombiel.settings,
        };

        if (options.devHost) {
          ombData.runserver = {hostname: options.devHost};
        }

        pkgjson.ombiel = ombData;
        pkgjson.name = `@ombiel/${packageName}`;
        pkgjson.version = "1.0.0";

        return fs.writeJSON(Path.resolve(destination,"package.json"),pkgjson,{spaces: 2})
        .then(()=>{

          process.chdir(Path.resolve(destination));
          log.info("Installing...");
          return yarn.command("install")
          .then(()=>{
            if (_.get(pkgjson,["scripts","aek:postcreate"])) {
              return yarn.command("run",["-s","aek:postcreate"]);
            }
            return Promise.resolve();
          });
        });
      });
    });
  },
};
