const log = require("@ombiel/aek-cli-support/logger");
const {FriendlyError} = require("@ombiel/aek-cli-support/nice-error");
const niceJson = require("@ombiel/aek-cli-support/nice-json");
const npm = require("@ombiel/aek-cli-support/npm");
const yarn = require("@ombiel/aek-cli-support/yarn");
// const config = require("@ombiel/aek-cli-support/config");
const utils = require("@ombiel/aek-cli-support/utils");
const pkgOmbiel = require("@ombiel/aek-cli-support/fstore")("./package.json","ombiel");

const _ = require("lodash");
const fs = require("fs");

const request = require("request");
const auth = require("@ombiel/aek-cli-support/auth");
const gulpTask = require("../gulp-task");

module.exports = {
  name: "deploy",
  help: "Deploy a package to your app",

  options: [
    {
      name: "username",
      message: "Please enter your App Manager username",
    },
    {
      name: "password",
      message: "Please enter your App Manager password",
      inputType: "password",
    },
    {
      name: "package",
      message: "Name of package to deploy (defaults to working directory)",
      position: 1,
    },
    {
      name: "hostname",
      message: "Which hostname would you like to target",
      beforeInquiry: function(opts,data) {
        return auth.getOptionHostnames(opts,data,pkgOmbiel.get("deployTarget"));
      },

      promptIfMissing: true,
    },

    {
      name: "version",
      message: "Version number or tag (defaults to 'latest')",
    },
    {
      name: "newversion",
      abbr: "n",
      flag: true,
      message: "Automatically bump version number",
    },
    {
      name: "appManagerBase",
      noPrompt: true,
      default: pkgOmbiel.get("appManager.baseURL"),
    },
    {
      name: "bypassSSLVerification",
      noPrompt: true,
      default: pkgOmbiel.get("appManager.bypassSSLVerification") || false,
    }
  ],

  callback: function(options) {

    function getPackageInfo(path) {
      let pjson = fs.readFileSync("./package.json");
      try {
        pjson = JSON.parse(pjson);
      }
      catch (e) {
        throw FriendlyError("Unable to parse package.json");
      }
      return _.get(pjson,path);
    }


    function doPreRelease({currentVersion,workingVersion}) {

      // newer boilerplate includes an "aekrelease" script to rework this part of the process and allow for hooks
      if (getPackageInfo(["scripts","aek:prerelease"])) {
        return yarn.command("run",["-s","aek:prerelease"]);
      }

      // legacy support for older projects with no aekrelease script
      return new Promise((resolve)=>{

        let version;

        gulpTask("_predeploy",function() {
          let versionPromise = Promise.resolve();
          workingVersion = getPackageInfo("version") || "0.0.1";
          version = workingVersion;
          if (currentVersion === workingVersion) {
            log.info("Bumping version number...");
            versionPromise = npm.command("version",["patch"],"pipe").then(function(newVersionNumber) {
              if (newVersionNumber.charAt(0) === "v") {
                newVersionNumber = newVersionNumber.substr(1);
              }
              log.info(`New version number is: ${newVersionNumber}`);
              version = newVersionNumber;
            });
          }

          versionPromise.then(function() {
            log.warn(`Pushing working package to registry (${version})`);
            resolve();
          });
        });
      });

    }



    return auth.getUserDetails(options)
    .then(function(userDetails) {

      let pkg = options.package;
      let version = options.version;

      let promise = Promise.resolve();

      if (!pkg) {

        log.info("No package specified");

        // check name
        const name = getPackageInfo("name");
        if (!name) {
          throw FriendlyError(`Package must have a name (eg {"name":"@ombiel/mypackage"})`);
        }

        if (name.indexOf("@ombiel") !== 0) {
          throw FriendlyError(`To push packages to the AEK registry, you must use the "ombiel" scope - please prefix your package name with "@ombiel" (eg {"name":"@ombiel/mypackage"})`);
        }
        pkg = name;

        log.info(`Using package from working directory: ${pkg}`);

        if (!version) {
          log.info("No version specified");
          promise = promise.then(function() {
            return new Promise(function(resolve,reject) {

              const workingVersion = getPackageInfo("version") || "0.0.1";
              log.info(`current working version: ${workingVersion}`);

              npm.view().then(function(data) {
                // console.log("data",data)
                const currentVersion = data ? data.version : null;
                const remoteVersions = data ? data.versions : [];
                const alreadyExists = _.includes(remoteVersions,workingVersion);

                if (currentVersion) {
                  log.info(`Latest registry version is: ${currentVersion}`);
                }

                if (!options.newversion && alreadyExists) {
                  log.warn("This package already exists in the registry");
                  log.warn("To deploy a new version, change the version number in package.json or use the '--newversion' option");
                  version = workingVersion;
                  resolve(true);
                }
                else {
                  doPreRelease({currentVersion,workingVersion}).then(()=>{
                    version = getPackageInfo("version");
                    log.warn(`Pushing working package to registry (${version})`);
                    yarn.command("publish").then(function() {
                      resolve(true);
                    });
                  })
                  .catch((err)=>{

                    if (err === 1) {
                      err = FriendlyError("Prerelease Script Failed");
                    }
                    reject(err);
                  });
                }
              });
            });
          });
        }
      }



      return promise.then(function() {
        if (!pkg) {
          throw FriendlyError("package undeterminable");
        }

        if (pkg.indexOf("@ombiel/") !== 0) {
          pkg = `@ombiel/${pkg}`;
        }

        if (!version && (pkg.substr(1).indexOf("@") === -1)) {

          version = "latest";
        }

        if (version) {
          if (version.indexOf("v") === 0) {
            version = version.substr(1);
          }
          if (version.indexOf("@") !== 0) {
            version = `@${version}`;
          }
          pkg += version;
        }

        log.info("Deploying package: ",pkg);

        return npm.view(pkg)
        .then(function(data) {

          if (!data) {
            throw FriendlyError(`Can't find package: ${pkg}`);
          }

          if (_.get(data,"ombiel.deployable") === false) {
            throw FriendlyError("This package has been flagged as non-deployable as it is designed for library use only");
          }

          log.info(`Package tarball is: ${data.dist.tarball}`);

          const dc = utils.getDC(userDetails.username);

          return new Promise((resolve,reject)=>{

            const requestOptions = {
              strictSSL: !options.bypassSSLVerification,
              form: {
                username: dc.username,
                token: userDetails.token,
                hostname: options.hostname,
                tarball: data.dist.tarball,
                name: data.name.split("/").slice(1).join("/"),
                version: data.version,
              }
            };

            request.post(`${options.appManagerBase || dc.appManagerBase}/aek/tools/deploy`,requestOptions,function(err,res,body) {

              if (err) {
                log.error("App Manager request failed",err);
              }

              try {
                data = JSON.parse(body);
              }
              catch (e) {
                log.error("Unable to parse App Manager response",e);
                console.log(body)
                return;
              }

              if (!data.details) {
                console.log(data)
                const message = data.msg ? `${data.msg}: ` : "";
                log.error(`${message}${_.isString(data.error) ? data.error : "Deployment Failed"}`);
                reject(data.error);
              }

              else {
                log.info("Deploy Successful","\n",niceJson({
                  details: _.omit(data.details,"token"),
                  savedScreens: data.saved_screens,
                  publicPaths: data.publicPaths,
                  savedActions: data.saved_actions,
                  tileComponents: data.saved_tile_components,
                }));
                resolve();
              }
            });
          });
        });
      });
    });
  },
};
